#include "X3D_GLBuilderRenderingVisitor.h"
#include "X3D_IndexedLineSet.h"
#include "X3D_PointSet.h"
#include "X3D_TriangleFanSet.h"
#include "X3D_TriangleSet.h"
#include "X3D_TriangleStripSet.h"
#include "GL_IndexedLineSet.h"
#include "GL_PointSet.h"
#include "GL_TriangleFanSet.h"
#include "GL_TriangleSet.h"
#include "GL_TriangleStripSet.h"
#include "X3D_X3DGeometryNode.h"

namespace X3DTK {
namespace X3D {

GLBuilderRenderingVisitor::GLBuilderRenderingVisitor()
: RenderingVisitor()
{
  // Enter functions.
  defineEnterFunction(&GLBuilderRenderingVisitor::enterIndexedLineSet);
  defineEnterFunction(&GLBuilderRenderingVisitor::enterPointSet);
  defineEnterFunction(&GLBuilderRenderingVisitor::enterTriangleFanSet);
  defineEnterFunction(&GLBuilderRenderingVisitor::enterTriangleSet);
  defineEnterFunction(&GLBuilderRenderingVisitor::enterTriangleStripSet);

  // WalkOn functions
  defineWalkOnFunction(&GLBuilderRenderingVisitor::walkOnX3DGeometryNode);
}

bool GLBuilderRenderingVisitor::walkOnX3DGeometryNode(X3DGeometryNode *, SFAbstractNode) 
{
  return false;
}

void GLBuilderRenderingVisitor::enterIndexedLineSet(IndexedLineSet *I) 
{
  GLBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
  GL::SFNode GI = stateVariables->getNode(I);
  if (GI == 0)
  {
    GI = new GL::IndexedLineSet();
    GI->setX3DReference(I);
    stateVariables->addCoupleNode(I, GI);
  }   
  stateVariables->pushNode(GI);
}

void GLBuilderRenderingVisitor::enterPointSet(PointSet *P) 
{
  GLBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
  GL::SFNode GP = stateVariables->getNode(P);
  if (GP == 0)
  {
    GP = new GL::PointSet();
    GP->setX3DReference(P);
    stateVariables->addCoupleNode(P, GP);
  }   
  stateVariables->pushNode(GP);
}

void GLBuilderRenderingVisitor::enterTriangleFanSet(TriangleFanSet *T) 
{
  GLBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
  GL::SFNode GT = stateVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::TriangleFanSet();
    GT->setX3DReference(T);
    stateVariables->addCoupleNode(T, GT);
  }   
  stateVariables->pushNode(GT);
}

void GLBuilderRenderingVisitor::enterTriangleSet(TriangleSet *T) 
{
  GLBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
  GL::SFNode GT = stateVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::TriangleSet();
    GT->setX3DReference(T);
    stateVariables->addCoupleNode(T, GT);
  }   
  stateVariables->pushNode(GT);
}

void GLBuilderRenderingVisitor::enterTriangleStripSet(TriangleStripSet *T) 
{
  GLBuilderStateVariables *stateVariables = GraphTraversal::getInstanceOf<GLBuilderStateVariables>();
  GL::SFNode GT = stateVariables->getNode(T);
  if (GT == 0)
  {
    GT = new GL::TriangleStripSet();
    GT->setX3DReference(T);
    stateVariables->addCoupleNode(T, GT);
  }   
  stateVariables->pushNode(GT);
}

}
}
